
function [C,S] = qq_als(X,C,K)
global metset
% function [C,S] = qq_als (X,C,K)
% Alternating regression from spectral start estimates
% X=raw data
% C=Chromatographic estimates
% S=Spectral estimates
% K=number of components

unimod=metset.quant_unimod;     % Unimodality in AR/ALS
posC=metset.quant_posC;       % Positive C in AR/ALS
posS=metset.quant_posS;       % Positive S in AR/ALS
closure=metset.quant_closure;    % Closure of C in AR/ALS
normS=metset.quant_normS;      % Normalize S in AR/ALS
stopcrit=metset.quant_stopcrit; % Convergence criterion

initnorm=norm(C); % Norm of C, used in convergence criterion

tic=sum(X,2); % Total ion current for closure

[m,n]=size(X);

niter=0; convergence=0;
while niter < metset.quant_niter & convergence==0
   niter=niter+1;
   oldC=C; %Used for convergence criterion 

   % Calculates S from C and X
   S=(X'/C')';
   
   % Positivity in S
   if posS S(find(S(:,:)<0))=0; end
   
   % Normaliser S her?
   %if normS for i=1:K fm(i,i)=100/norm(S(i,:)); end; S=fm*S; end % Sets norm=100
   %f=max(S,[],2); fm=diag(100./f); S=fm*S; % Sets max=100
   if normS f=sum(S,2); fm=diag(1./f); S=fm*S; end % Sets sum=1
   
   % Calculates C from X and S
   C=X/S;
      
   % Negative elements in C set to 0
   if posC C(find(C(:,:)<0))=0; end    
   
   % Unimodality in C
   if unimod
       for i=1:K
          maxc=max(C(:,i));
          mapos=find(C(:,i)==maxc);  % Estimates position for max
          for j=mapos:-1:2
             if C(j-1,i)>C(j,i)
                C(j-1,i)=C(j,i);
             end
          end
             for j=mapos:m-1
             if C(j+1,i)>C(j,i)
                C(j+1,i)=C(j,i);
             end
          end
       end
   end
   
   % Closure of C (by addition of missing parts of X)
   if closure      
       newtic=sum((C*S),2); 
       dtic=(newtic-tic)/K;       
       C=C+repmat(dtic,1,K); 
   end
   
   diffC=C-oldC; % Calculates difference matrix of C
   normdiff=norm(diffC); % norm of difference matrix
   if log10(initnorm/normdiff)>stopcrit convergence=1; end % Compares norm of initial C with norm of difference matrix
     
end % while

qq_message(1,['|b| ALS: number of iterations: ' num2str(niter) ',  Stopvalue: ' num2str(ceil(log10(initnorm/normdiff)))])  

