
function [ipure] = qq_borgen (X, K, plotborgen, llabels)

% function [C,S,ipure] = borgen (X, K, plotborgen)
% Returns pure variables after modified borgen method
% Grande & Manne, Chemom. Int. Lab. Syst. 50 (2000) 19-33
% X = Raw data
% K = number of components
% plotborgen = 1 gives Borgen plot
% C and S, Estimated chromatographic/spectral profiles
% ipure = column number for the pure variables
% purevar = pure variables

global positions

maxrank=min(size(X));
if K>maxrank, K=maxrank; end  % Setter K lik 1 hvis rank > antall ioner/scan
%if K==0, K=1; end         % Setter K lik 1 hvis rank =0

if K<2
    sumX=sum(X); [maxX,ipure]=max(sumX); ipure=ipure(1); 
else
    kappa = 1;

    [i1,j1] = size(X);
    ipure = zeros(1,K);

    % Truncation of X to rank K
    [U,d,V] = svd(X);
    if sum(U(:,1))<0;
       U(:,1)=-U(:,1);
       V(:,1)=-V(:,1);
    end;
    U=U(:,1:K); d=d(1:K,1:K); V=V(:,1:K);

    % Ridge parameter
    [rp,ind] = max(d(1,1)*V(:,1));
    rp=kappa*rp;

    % Normalization
    % Size of Y-matrix is (K-1)*j1
    N=d*V';
    Y=N(2:K,:)./(ones(K-1,1)*(N(1,:)+rp*ones(1,j1)));

    % Normalized Y is saved to be used in the Borgen plot
    YB=Y;

    % Step 1. First pure variable
    if K==2
       [r,ipure(1)]=max(Y.^2);
    else
       [r,ipure(1)]=max((sum(Y.^2))');
    end

    % Step 2. Second pure variable
    Y = Y-Y(:,ipure(1))*ones(1,j1);

    if K == 2
       [r,ipure(2)]=max(Y.^2);
    else
       [r,ipure(2)]=max((sum(Y.^2))');
    end

    % Step 'n', Third and larger
    for k = 3:K;
       e = Y(:,ipure(k-1));
       Y = Y - e*((e'*Y)/(e'*e));
       [r,ipure(k)] = max((sum(Y.^2))');
    end
end

ipure=sort(ipure);

% -------------------------------------------------------------------------
% --------------------------------- PLOT ----------------------------------
% -------------------------------------------------------------------------

if plotborgen==1
   subplot('position',positions.smallplot); plot(0);
   if K<2;
      text(1,0,'Rank < 2','Fontsize',25,'FontWeight','bold','HorizontalAlignment','center')
   end

   % Two components
   if K == 2
      hold on
      for j = 1:j1
         a1 = d(1,1)*(V(j,1)); 
         a2 = d(2,2)*V(j,2);
         plot(1,(a2*(1+rp)/(a1+rp)),'.')
      end
   
      for k = 1:K
         a1=d(1,1)*V(ipure(k),1);
         a2=d(2,2)*V(ipure(k),2);
         x1=-rp; y1=0; x2=1;
         y2=a2*(1+rp)/(a1+rp);
         text(1,y2,['  ' num2str(llabels(ipure(k)))],'fontsize',10);
         line([x1 x2],[y1 y2])
         plot(x2,y2,'.r','MarkerSize',20);
      end
      hold off
   end
       
   % More than two comp
   if K>2
      plot(YB(1,:),YB(2,:),'-.')
      hold on
      for j=1:K
         plot(YB(1,ipure(j)),YB(2,ipure(j)),'.r','MarkerSize',20)
      end
      for j=1:j1
         text(YB(1,j),YB(2,j),['  ' num2str(llabels(j))],'fontsize',8,'HorizontalAlignment','left')
      end
      hold off
   end
end





      





