function [dels,keeps,mcq] = coda(X,window,level);
%function [Xcut,keeps,mcq] = coda(X,window,level);
%
%Component detection algorithm for MS data
%Windig, Phalp & Payne, Analytical Chemistry, 1996, 68, 3602-3606
%
%Original version published in the paper. Transcribed and modified by Bjrn
%Grung 18th september 2009
%Modified to fit Q, SAM 2011
%
%Input:
%X is the data matrix. Each row is a mass spectrum. 
%window is the window size used for smoothing. Must be an odd number.
%Recommended value is 5.
%level is the lower level for the MCQ limit. Recommended level is 0.89. 
%
%Output:
%mcq is a vector containg the MCQ for each mass in the original data set.
%keeps is a vector containing the position in X of the masses kept after
%CODA. 
%Xcut is data matrix that results when all noisy masses are removed.

[n,m] = size(X);

window = round(window);

if window < 3
    window = 5;
    disp('Window size set to 5')
end

if mod(window,2) == 0
    window = round(window) + 1;
end

halfwindow = (window-1)/2;
Xsmooth = zeros(n-window+1,m);

%smoothing
for i = 1:window
    Xsmooth = Xsmooth + X(i:(n-window+i),:);
end

%truncate X so that it has the same size as Xsmooth
X([1:halfwindow n-halfwindow+1:n],:) = [];
n = n - window + 1;

%normalise masses to unit length
lengthdata = sqrt(sum(X.*X));
Xnorm = X./lengthdata(ones(1,n),:);

%standardise smoothed data
meansmooth = mean(Xsmooth);
stdsmooth = std(Xsmooth);
Xsmoothstand = (Xsmooth-meansmooth(ones(1,n),:))./stdsmooth(ones(1,n),:);

%calculate MCQ index
mcq = (1/sqrt(n-1))*sum(Xnorm.*Xsmoothstand);

dels = find(mcq<level);
keeps = find(mcq>=level);
%Xcut = X(:,keeps);
%plotting the results

%figure
%subplot(311), plot(sum(X'))
%title('raw data TIC')
%subplot(312), plot(sum(X(:,(mcq>level))'))
%title('CODA TIC')
%subplot(313), plot(X(:,(mcq>level)))
%title('MC''s CODA')
