function  [C,S] = qq_gentle (X,C,negpar,refinepar)

% disp('----------- INITIATE -----------')

[n_scans, n_analytes]=size(C);

% Strategy for selection of peak j 
% 1: peak-j is peak with largest value over minima in peak-i
% 2: peak-j is neighbour peak (i-1 or i+1) with largest value over minima
% of peak i (Gentle default)
% 3: peak j is peak with most similar spectrum to peak i.

S=X'/C'; S=S';

if negpar>0
    [C,S]=negative(C,S,negpar);
end

if refinepar==1
    [C,S]=refine1(C,S); % grense for elution vindu er minpkt under k
end

if refinepar==2
    [C,S]=refine2(C,S); % grense for elution vindu er under k
end

if refinepar==3
    [C,S]=refine3(C,S); % grense for elution vindu er under k
end

% -----------------------------------------------------------------
%               S U B F U N C T I O N S
%------------------------------------------------------------------



% --------------------- NEGATIVE ----------------------------------
% NEGATIVE - removes negative peaks

function [C,S]=negative(C,S,selectstrat)

% disp('------- Remove negative -------')

[n_scans, n_analytes]=size(C);

% Beregner k til 1%% av maxC
maxC=max(max(C)); k=maxC/100;

niter=50; pp=0; count=50;
while pp<niter
    pp=pp+1;
    konvergens=1;

    % finner index og mest negative element i C
    
    minminC=min(min(C));              % minima i c                         
    if abs(minminC)>k  % Tilfredstiller krav 1, 1% av k 
       % disp('------ krav 1 ------')

       [rti,ii]=find(C==minminC); % rti: retensjonsindex til min, ii, analytt nr.

       valuesatmin=C(rti,:); % alle verdier over rti
       % Finner index til topp det skal subtraheres fra.
       [maxovermin, jj2]=max(C(rti,:)); % jj2= index til c med strste verdi over rti
       % bestemmer jj
       
       if selectstrat==1; jj=jj2; end % Tester alle topper
       
       if selectstrat==2 % tester nabotopper (Som i Gentle)
           if ii>1 & ii<n_analytes, % dersom ii ikke er frste eller siste topp
               if valuesatmin(ii-1)>valuesatmin(ii+1), jj=ii-1; else jj=ii+1; end % velger nabotopp med hyest verdi over rti
           end
           if ii==1, jj=ii+1; end
           if ii==n_analytes  jj=ii-1;  end
       end
       
       if selectstrat==3 % Benytter covar. mellom spektra.
           corrl=corrcoef(S'); corrl=corrl(:,ii); corrl(ii)=0;[corrl, jj]=max(corrl);
       end 
       
       if valuesatmin(jj)>k % Krav 2 (verdi for jj hyere enn k)
          % disp('----- Krav 2 ----- oppdaterer -----')
          konvergens=0;
          C(:,ii)=C(:,ii)+k*C(:,jj);
          S(jj,:)=S(jj,:)-k*S(ii,:);
      end  
    end
    if konvergens count=pp; pp=niter; 
        % disp('---------konvergens--------'); 
    end     
end

% txt=['---- N.iterations: ' num2str(count) '------']; disp(txt)





% --------------------- REFINE1 ----------------------------------
% REFINE1 - refines solution from 'negative'

function [C,S]=refine1(C,S)

% disp('-------------- GLATTING ---------------')
[n_scans, n_analytes]=size(C);

% Beregner k til 1%% av maxC
maxC=max(max(C)); k=maxC/100;


niter=50; pp=0; count=50;
while pp<niter
    pp=pp+1;
    
    [maxC, maxi]=max(C); % Maxi inneholder analyttenes maxima
    
    largestmax=0; maxv=0;
    for m=1:n_analytes
        lok=[];
        for n=maxi(m):-1:2 % Fra max til start av chrom
            if (C(n-1,m)>C(n,m) & C(n,m)<k)|C(n,m)<0, lok=[lok n]; end
        end
        maxv=0;
        if isempty(lok)==0, minp=max(lok); [maxv,maxind]=max(C(1:minp,m)); end  
        if maxv>largestmax; largestmax=maxv; ii=m; ii_ma_i=maxind; ii_mi_i=minp; end % ii, max index, minpkt ind.
        lok=[];
        for n=maxi(m):n_scans-1 % Fra max til slutt av chrom
            if (C(n+1,m)>C(n,m) & C(n,m)<k)|C(n,m)<0, lok=[lok n]; end
        end     
        maxv=0;
        if isempty(lok)==0, minp=min(lok); [maxv,maxind]=max(C(minp:n_scans,m)); 
            maxind=maxind+minp-1; end  
        if maxv>largestmax; largestmax=maxv; ii=m; ii_ma_i=maxind; ii_mi_i=minp; end % ii, max index, minpkt ind.    
    end
    
    % finner jj (kan ikke vre samme som ii)
    v_rt=(C(ii_ma_i,:)); % v_rt inneholder all verdier ved ii_ma_i
    [testmax,testmax_c]=max(v_rt); if testmax_c==i message('Double peaks present, too low rank?'); end
    v_rt(ii)=[]; % sletter ii fra v_ind
    [jj_i, jj]=find(C==max(v_rt)); % Finner jj. jj_i skal vre lik ii_ma_i.
    %(alternativ sett v_rt(ii) lik 0, pp er v_rt med hyest absoluttverdi)
    
    % Start og slutt for regresjonsvektorer
    if ii_ma_i > ii_mi_i start=ii_mi_i; slutt=n_scans; else start=1; slutt=ii_mi_i; end
    % Regresjonsvektorer og beregning av b.
    ii_vect=C(start:slutt,ii);
    jj_vect=C(start:slutt,jj);    
    b=ii_vect'/jj_vect'; % lokal beregning av k

    % ----------- Transformasjoner p C og S --------------
    C(:,ii)=C(:,ii)-b*C(:,jj);
    S(jj,:)=S(jj,:)+b*S(ii,:);
    
    if b<0.0000001; pp=niter; end %(konvergens)
    
end % while
    

    

% --------------------- REFINE2 ----------------------------------
% REFINE2 - refines solution from 'negative'

function [C,S]=refine2(C,S)

% disp('-------------- GLATTING ---------------')
[n_scans, n_analytes]=size(C);

% Beregner k til 1%% av maxC
maxC=max(max(C)); k=maxC/100;


niter=50; pp=0; count=10;
while pp<niter
    pp=pp+1;
    
    %--------------- NY ----------------
    [maxC, maxi]=max(C); % Maxi inneholder analyttenes maxima
    
    largestmax=0; maxv=0;
    for m=1:n_analytes
        lok=[];
        for n=maxi(m):-1:2 % Fra max til start av chrom
            if C(n,m)<k, lok=[lok n]; end                  % FORSKJELL TIL 'REFINE-1' HER
        end
        maxv=0;
        if isempty(lok)==0, minp=max(lok); [maxv,maxind]=max(C(1:minp,m)); end  
        if maxv>largestmax; largestmax=maxv; ii=m; ii_ma_i=maxind; ii_mi_i=minp; end % ii, max index, minpkt ind.
        lok=[];
        for n=maxi(m):n_scans-1 % Fra max til slutt av chrom
            if C(n,m)<k, lok=[lok n]; end                  % FORSKJELL TIL 'REFINE-1' HER
        end     
        maxv=0;
        if isempty(lok)==0, minp=min(lok); [maxv,maxind]=max(C(minp:n_scans,m)); 
            maxind=maxind+minp-1; end  
        if maxv>largestmax; largestmax=maxv; ii=m; ii_ma_i=maxind; ii_mi_i=minp; end % ii, max index, minpkt ind.    
    end
    
    % finner jj (kan ikke vre samme som ii)
    v_rt=(C(ii_ma_i,:)); % v_rt inneholder all verdier ved ii_ma_i
    [testmax,testmax_c]=max(v_rt); if testmax_c==i message('Double peaks present, too low rank?'); end
    v_rt(ii)=[]; % sletter ii fra v_ind
    [jj_i, jj]=find(C==max(v_rt)); % Finner jj. jj_i skal vre lik ii_ma_i.
    %(alternativ sett v_rt(ii) lik 0, pp er v_rt med hyest absoluttverdi)
    
    % Start og slutt for regresjonsvektorer
    if ii_ma_i > ii_mi_i start=ii_mi_i; slutt=n_scans; else start=1; slutt=ii_mi_i; end
    % Regresjonsvektorer og beregning av b.
    ii_vect=C(start:slutt,ii);
    jj_vect=C(start:slutt,jj);    
    b=ii_vect'/jj_vect'; % lokal beregning av k

    % ----------- Transformasjoner p C og S --------------
    C(:,ii)=C(:,ii)-b*C(:,jj);
    S(jj,:)=S(jj,:)+b*S(ii,:);
    
    if b<0.0000001; pp=niter; end %(konvergens)
    
end % while 


% --------------------- REFINE3 ----------------------------------
% REFINE1 - refines solution from 'negative'

function [C,S]=refine3(C,S)

disp('-------------- GLATTING ---------------')
[n_scans, n_analytes]=size(C);

% Beregner k til 1%% av maxC
maxC=max(max(C)); k=maxC/100;


niter=50; pp=0; count=50;
while pp<niter
    pp=pp+1;
    
    [maxC, maxi]=max(C); % Maxi inneholder analyttenes maxima
    
    largestmax=0; maxv=0;
    for m=1:n_analytes
        lok=[];
        for n=maxi(m):-1:2 % Fra max til start av chrom
            if (C(n-1,m)>C(n,m) & C(n,m)<k)|C(n,m)<0, lok=[lok n]; end
        end
        maxv=0;
        % Forskjell til refine 1: Tester absolutt-verdier av C i flgende linje:
        if isempty(lok)==0, minp=max(lok); [maxv,maxind]=max(abs(C(1:minp,m))); end  
        if maxv>largestmax; largestmax=maxv; ii=m; ii_ma_i=maxind; ii_mi_i=minp; end % ii, max index, minpkt ind.
        lok=[];
        for n=maxi(m):n_scans-1 % Fra max til slutt av chrom
            if (C(n+1,m)>C(n,m) & C(n,m)<k)|C(n,m)<0, lok=[lok n]; end
        end     
        maxv=0;
        % Forskjell til refine 1: Tester absolutt-verdier av C i flgende linje:
        if isempty(lok)==0, minp=min(lok); [maxv,maxind]=max(abs(C(minp:n_scans,m))); 
            maxind=maxind+minp-1; end  
        if maxv>largestmax; largestmax=maxv; ii=m; ii_ma_i=maxind; ii_mi_i=minp; end % ii, max index, minpkt ind.    
    end
    
    % finner jj (kan ikke vre samme som ii)
    v_rt=(C(ii_ma_i,:)); % v_rt inneholder all verdier ved ii_ma_i
    [testmax,testmax_c]=max(v_rt); if testmax_c==i message('Double peaks present, too low rank?'); end
    v_rt(ii)=[]; % sletter ii fra v_ind
    [jj_i, jj]=find(C==max(v_rt)); % Finner jj. jj_i skal vre lik ii_ma_i.
    %(alternativ sett v_rt(ii) lik 0, pp er v_rt med hyest absoluttverdi)
    
    % Start og slutt for regresjonsvektorer
    if ii_ma_i > ii_mi_i start=ii_mi_i; slutt=n_scans; else start=1; slutt=ii_mi_i; end
    % Regresjonsvektorer og beregning av b.
    ii_vect=C(start:slutt,ii);
    jj_vect=C(start:slutt,jj);    
    b=ii_vect'/jj_vect'; % lokal beregning av k

    % ----------- Transformasjoner p C og S --------------
    C(:,ii)=C(:,ii)-b*C(:,jj);
    S(jj,:)=S(jj,:)+b*S(ii,:);
    
    if b<0.0000001; pp=niter; end %(konvergens)
    
end % while


% -------------- MESSAGE -------------------
function message(txt)
MessageTxt=findobj('Tag','MessageTxt');
set(MessageTxt,'String',txt);