function qq_load_cdf(an)

%global D %X %llabels %lions %pathinfo %Info % Change to persistent
% CDF load Requires the mex-file iCDF.mexw32 from http://www.models.life.ku.dk/iCDF
% D = Directory data, 
% loadset.X = Loaded data
% loadset.rt / loadset.ions = local labels/ions (before accept)

global XR labels ions metset fileset loadset locset

qq_message(2)

if nargin<1
   an=0;
end

if an==0
    openwin
    pathsearch;
    loadset.pathinfo=getfiles;
    loadset.X=[]; loadset.rt=[]; loadset.ions=[];
    qq_message(1,'CDF load Requires the mex-file iCDF.mexw32 from http://www.models.life.ku.dk/iCDF')
end

if an==1
    pathsearch;
    loadset.pathinfo=getfiles;
end
if an==1.1
    str=get(findobj('Tag','SearchPathEdit'),'String'); locstr=locset.path_cdf; 
    if strmatch(locstr,str)==1 && isdir(str)
        metset.path_cdf=str(length(locstr)+2:length(str)); qq_message(1,'|b|Path updated in method');
    else
        qq_message(1,'|r|Path does not match local settings');
    end
end

% Callback fra Dirlist
if an==2.3 
    loadset.pathinfo=getfiles;
end

% Load
if an==3.1
    h=findobj('Tag','ContentsList'); val=get(h,'value');
    
    [loadset.X,loadset.rt,loadset.ions,loadset.Info]=loadcdf(loadset.pathinfo(val).path,loadset.pathinfo(val).name);
        
    [loadset.X,loadset.rt,loadset.ions,loadset.Info]=truncate(loadset.X,loadset.rt,loadset.ions,loadset.Info); 
    qq_load_plots(loadset.X,loadset.rt,loadset.ions,loadset.Info,1);
end

% Export
if an==3.2
    h=findobj('Tag','ContentsList'); val=get(h,'value');
    export(loadset.pathinfo(val),[locset.path_qqraw '\' metset.path_qqraw ]);
end
if an==3.3
    exportall(loadset.pathinfo,[locset.path_qqraw '\' metset.path_qqraw ]);
end

% Plots
if an==4.1 & isempty(loadset.X)==0 qq_load_plots(loadset.X,loadset.rt,loadset.ions,loadset.Info,1); end
if an==4.2 & isempty(loadset.X)==0 qq_load_plots(loadset.X,loadset.rt,loadset.ions,loadset.Info,2); end

% 
if an==10
   dim=size(loadset.X);
   if dim(2)>0   
      initialize(loadset.X,loadset.rt,loadset.ions,loadset.Info)
      qq_closewin; 
   else
      qq_message(1,'|r|No data'); 
   end
end
        

%======================================== SUBFUNCTIONS ========================================

% -------------------- LOADCDF --------------------------
function [X,rt,ions,Info]=loadcdf(loadpath,name)

pathname=[loadpath '\' name '.cdf']

data = iCDF(pathname);

% reads data, adapted from iCDF_load, www.models.life.ku.dk./iCDF
X = zeros(length(data.variables.scan_index.data),max(round(data.variables.mass_values.data)));   
for i = 1:length(data.variables.scan_index.data)-1
    X(i,round(data.variables.mass_values.data(data.variables.scan_index.data(i)+1:data.variables.scan_index.data(i+1))))......
        = data.variables.intensity_values.data(data.variables.scan_index.data(i)+1:data.variables.scan_index.data(i+1));
end
rt=(data.variables.scan_acquisition_time.data/60)';
ions=1:round(max(data.variables.mass_range_max.data));
     
Info.File_Name=name;
Info.Operator=data.attributes.operator_name;
Info.Time=convtime(data.attributes.experiment_date_time_stamp);
Info.Instrument=data.variables.instrument_name.data;
Info.Method='Unknown';
Info.Note=data.attributes.experiment_title;
if isfield(data.attributes,'source_file_reference') Info.SourceFile=data.attributes.source_file_reference; end
if isfield(data.attributes,'source_file_format') Info.SourceFormat=data.attributes.source_file_format; end
Info.IonizationMode=data.attributes.test_ionization_mode;
Info.Polarity=data.attributes.test_ionization_polarity;
Info.Source='CDF import';


% -------------------- CONVTIME --------------------------
function outstr=convtime(instr)

yr=str2double(instr(1:4));
mo=str2double(instr(5:6));
dy=str2double(instr(7:8));
hr=str2double(instr(9:10));
mi=str2double(instr(11:12));
se=str2double(instr(13:14));

outstr=datestr([yr mo dy hr mi se]);


% --------------- TRUNCATE -------------------------------
function [X,llabels,lions,Info]=truncate(X,llabels,lions,Info)
global metset

str=['|b|loaded: ' num2str(length(llabels)) ' scans, ' num2str(length(lions)) ' ions'];

% Remove n lowest ions (max value)
nremoved=0;
maxv=max(X); 
%LowEdit=findobj('Tag','LowEdit');
%nlow=str2double(get(LowEdit,'String'));
nlow=metset.load_dellow;
if nlow>=length(maxv); nlow=length(maxv)-1; end
[dummy,ind]=sort(maxv); delvect=ind(1:nlow); 
X(:,delvect)=[]; lions(delvect)=[]; nremoved=nremoved+length(delvect);

% Remove ions with max below treshold
maxv=max(X);
%TreshEdit=findobj('Tag','TreshEdit');
%tresh=str2double(get(TreshEdit,'String'));
tresh=metset.load_iontresh;
%if isnan(tresh)==1 tresh=0; end
delvect=find(maxv<tresh); 
if length(delvect)==length(maxv); tresh=max(maxv); delvect=find(maxv<tresh); end
X(:,delvect)=[]; lions(delvect)=[]; nremoved=nremoved+length(delvect);

if nremoved>0 str=[str ',  ' num2str(nremoved) ' ions truncated'];  end
qq_message(1,[str,'.']);
    

% -------------- SUBFUNCTION EXPORTALL -------------------
function exportall(pathinfo,exportpath)
global colors

winpar={'M-Win','Batch export',[0.1 0.5 0.8 0.4]};
p=0; p=p+1; [ui{p,1:7}] = deal('','listbox',[1 1 1],[0.05 0.1 0.2 0.7],'',2,'RemList');
p=p+1; [ui{p,1:7}] = deal('Remaining','Text',colors.bc,[0.05 0.8 0.2 0.1],'',2,'');
p=p+1; [ui{p,1:7}] = deal('','listbox',[1 1 1],[0.3 0.1 0.2 0.7],'',2,'ConvList');
p=p+1; [ui{p,1:7}] = deal('Converted','Text',colors.bc,[0.3 0.8 0.2 0.1],'',2,'');
p=p+1; [ui{p,1:7}] = deal('','Text',colors.bc,[0.55 0.4 0.3 0.5],'',3,'ConvInfo');
level='';
h0=qq_drawwin(winpar,ui,level);

RemList=findobj(h0,'Tag','RemList'); ConvList=findobj(h0,'Tag','ConvList'); ConvInfo=findobj(h0,'Tag','ConvInfo');

n=length(pathinfo);
dirname=['\QQ-Raw-' datestr(now,1)];
a=dir([exportpath '\' dirname]); if isempty(a) mkdir(exportpath,dirname); end

starttime=now;
for x=1:n   
    RemStr=strvcat(pathinfo(x:n).name); set(RemList,'String',RemStr);
    IStr{1}=['Now Converting: ' pathinfo(x).name]; 
    IStr{2}=[num2str(x-1) ' Converted, ' num2str(n-x+1) ' Remaining']; IStr{3}=['------------------------------------------------']; 
    ConvStr=strvcat(pathinfo(1:x-1).name); set(ConvList,'String',ConvStr);
    set(ConvInfo,'String',IStr);
    
    export(pathinfo(x),[exportpath '\' dirname]);
    
    elapsed=now-starttime; IStr{4}=['Elapsed time: ' datestr(elapsed,13)]; 
    per_conv=elapsed/x;
    n_rem=n-x;
    remtime=per_conv*n_rem;
    IStr{5}=['Est rem. time: ' datestr(remtime,13)];
    IStr{6}=['Est. end time: ' datestr(now+remtime,13)];  
    set(ConvInfo,'String',IStr); 
end

qq_message(1,['|b|Data exported to separate directory: ' exportpath dirname])
close(h0);

% -------------- SUBFUNCTION EXPORT -------------------
function export(pathinfo,exportpath)

[XR,rt,ions,Info]=loadcdf(pathinfo.path,pathinfo.name);

[XR,rt,ions,Info]=truncate(XR,rt,ions,Info); 

qq_message(1,['|b|Exporting: ' pathinfo.name]); drawnow
fname=['QQ_RAW_' pathinfo.name,'.mat'];
pname=[exportpath,'\',fname];

Objects=num2str(rt','%11.4f');
[m,n]=size(Objects); bl=repmat(' ',m,32-n); Objects=[Objects bl];

Variables=num2str(ions','%11.4f');
[m,n]=size(Variables); bl=repmat(' ',m,32-n); Variables=[Variables bl];

%save(pname,'XR','Objects','Variables','Info'); 
qq_savefile(pname,XR,Objects,Variables,Info,'-o -s')

qq_message(1,['|b|Saved as: ',fname,' in: ',exportpath]);

% --------------- GETFILES -------------

function pathinfo=getfiles
global metset

pathinfo(1).name=''; pathinfo(1)=[];

dirlist=findobj('Tag','DirList'); val=get(dirlist,'Value');
%if metset.file_showall==1
%    pathlist=get(dirlist,'String');
%else
    str=get(dirlist,'String'); pathlist=str(val);
    %end
    
searchstr=(get(findobj('Tag','SearchEdit'),'String'));
p=0;
for x=1:length(pathlist)
    DD=dir([pathlist{x} '\' searchstr]); le=length(DD);
    for y=1:le
        p=p+1; names{p}=DD(y).name(1:length(DD(y).name)-4); filepath{p}=pathlist{x};  
    end
end

ind=[]; if exist('names') [dummy,ind]=sortrows(strvcat(names)); end
for x=1:length(ind)
    pathinfo(x).name=names{ind(x)};
    pathinfo(x).path=filepath{ind(x)};
end

h=findobj('Tag','ContentsList'); 
set(h,'String',strvcat(pathinfo.name),'Value',1); 

% --------------- PATHSEARCH -------------
% Scanner path og subdirectories, sjekker om directories inneholder
% chemstation dirs.
function pathsearch
global metset

h=findobj('Tag','SearchPathEdit'); pathstr=get(h,'String');
h=findobj('Tag','SearchEdit'); searchstr=get(h,'String');

searchcont=0; if isempty(strmatch(searchstr,'')) searchcont=1; end

listlength=1; current=0; 
scanlist{1}=pathstr; 
dirlist{1}='';
DD=dir(pathstr); if isempty(DD) qq_message(1,'|r|Not a directory'); current=1; end
% SCAN
while current<listlength
    current=current+1;
    DD=dir(scanlist{current});
    isdir=find(cat(2,DD.isdir));
    q=0;
    for x=1:length(isdir)
        add=1; name=DD(x).name;
        
        % Directories som ikke skal sjekkes
        if strmatch(name,'.'); add=0; end
        if strmatch(name,'..'); add=0; end
        le=length(name); 
        %if le>1 & strmatch(name(le-1:le),'.D') add=0; end % Chemstation data directories
        if le>3 & strmatch(name(le-3:le),'.LOG') add=0; end % Log directories
        if le>3 & strmatch(name(le-3:le),'.LNK') add=0; end % Links
        
        if add==1 
            q=q+1;     
            listname=strrep([scanlist{current} '\' name],'/','\'); listname=strrep(listname,'\\','\');
            scanlist{q+listlength}=listname; 
        end
    end
    listlength=length(scanlist);
end % while

% Constrain
tom=[];
if searchcont==1
    for x=1:listlength
        dirstr=[scanlist{x} '\' searchstr]; 
        cont=dir(dirstr);
        tom(x)=isempty(cont);
    end
end
scanlist(find(tom))=[]; listlength=length(scanlist); 

% Sort
[A,ind]=sortrows(strvcat(scanlist)); 
for x=1:listlength
    dirlist{x}=scanlist{ind(x)};
end

h=findobj('Tag','DirList'); 
if isempty(dirlist) set(h,'String','Empty''Value',1); else set(h,'String',dirlist,'Value',1); end

% -------------- INITIALIZE -------------------
function initialize(X,rt,ions,Info)

global RE CL XR XBL fileset %labels ions

fn=fieldnames(fileset); fileset=rmfield(fileset,fn);

fileset.global_filename=Info.File_Name;
fileset.file_type='AIA NetCDF';

%fileset.bl_subtracted=0; % 1 hvis baselinjesubtrahert
%fileset.peak_lockvector=[]; % lst omrde
%fileset.quant_areas=0;        % Number of quantified areas
%fileset.global_currentcl=0;   % Current cluster
%fileset.global_nuclusters=0;  % Number of clusters
%fileset.global_nuquant=0;     % Number of quantified clusters
%fileset.results_npeaks=0;     % Number of peaks in results

XR=X; XBL=X; CL=[]; RE=[];

fileset.rt=rt;
fileset.ions=ions;
fileset.info=Info;




%======================================== WINDOW ========================================
% ------------------- OPENWIN ---------------------
function openwin

global positions colors metset locset

if isfield(metset,'load_iontresh')==0 metset.load_iontresh=0; end
if isfield(metset,'load_dellow')==0 metset.load_dellow=0; end
searchpath=[locset.path_cdf '\' metset.path_cdf]; 

bw=positions.bw;
bh=positions.bh;
eh=positions.eh; eo=(bh-eh)/2; % edit offset
gh=positions.gh;% grid height

bo=bw/5;
x=positions.gl:positions.gw:1; % horisontalt grid
y=positions.gt+0.5:-positions.gh:0; % vertikalt grid

winpar{1}='LoadCDFWin'; winpar{2}='Q: Load CDF';

% 'String','Style','Backgroundcolor','Position','Callback','Fonts'
p=0; o=bw/2;
% CLOSE
p=p+1;[ui{p,1:7}] = deal('Close','Pushbutton',colors.close,[x(1) y(20) bw bh*1.5],'qq_closewin',3,'CloseBtn');
p=p+1;[ui{p,1:8}] = deal('Accept','Pushbutton',colors.accept,[x(1) y(19)+bh/2 bw bh*1.5],'qq_load_cdf(10)',3,'AcceptBtn','bm');
% Path, lister, og file ext.
p=p+1;[ui{p,1:7}] = deal('','text',[1 1 1],[0 y(12) 1 bh],'',3,'');
p=p+1;[ui{p,1:7}] = deal('Search Path:','text',[1 1 1],[x(1) y(12)+eo bw*2 eh],'',3,'');
p=p+1;[ui{p,1:7}] = deal(searchpath,'edit',[1 1 1],[x(2)+bo y(12)+eo bw*5-bo eh],'qq_load_cdf(1)',2,'SearchPathEdit');
p=p+1;[ui{p,1:7}] = deal('  For:','text',[1 1 1],[x(7) y(12)+eo bw eh],'',3,'');
p=p+1;[ui{p,1:7}] = deal('*.cdf','edit',[1 1 1],[x(8)-bw/2 y(12)+eo bw*1.5 eh],'qq_load_cdf(1)',2,'SearchEdit'); % Sjekk Call
p=p+1;[ui{p,1:7}] = deal('SM','pushbutton',[1 1 1],[x(7)-bw/4 y(12)+eo bw/4 eh],'qq_load_cdf(1.1)',2,'');

% FilesList
p=p+1;[ui{p,1:7}] = deal('Searching...','listbox',[1 1 1],[x(2)+bo y(20) bw*5-bo y(2)-y(9)+bh],'qq_load_cdf(2.3)',2,'DirList');
p=p+1;[ui{p,1:7}] = deal('','listbox',[1 1 1],[x(8)-bw/2 y(20) bw*1.5 y(2)-y(9)+bh],'',2,'ContentsList');
% Get
%p=p+1;[ui{p,1:7}] = deal('All ->','Togglebutton',colors.accept,[x(7) y(14) bw/2 bh],'qq_load_cdf(2.1)',2,'GetToggle');
%p=p+1;[ui{p,1:7}] = deal('Sel.->','Togglebutton',colors.accept,[x(7) y(15) bw/2 bh],'qq_load_cdf(2.1)',2,'GetToggle');
% Load
p=p+1;[ui{p,1:8}] = deal('Load','Pushbutton',colors.btn{1},[x(9) y(14) bw bh],'qq_load_cdf(3.1)',2,'','-bm');
p=p+1;[ui{p,1:8}] = deal('Export','Pushbutton',colors.btn{6},[x(9) y(15) bw bh],'qq_load_cdf(3.2)',2,'','bm');
p=p+1;[ui{p,1:8}] = deal('Export All','Pushbutton',colors.btn{6},[x(9) y(16) bw bh],'qq_load_cdf(3.3)',2,'','bm');

% Treshold
p=p+1;[ui{p,1:7}] = deal('Ion tresh.','Text',colors.bc,[x(9)+bw*1/4 y(17)-eh*0.7 bw bh],'',2,'');
p=p+1;[ui{p,1:7}] = deal(metset.load_iontresh,'Edit',[1 1 1],[x(9)+bw*1/4 y(18) bw*3/4 eh],'qq_eval(''pn0'')',2,'metset.load_iontresh');
p=p+1;[ui{p,1:7}] = deal('Del. Lowest','Text',colors.bc,[x(9)+bw*1/4 y(19)-eh*0.7 bw bh],'',2,'');
p=p+1;[ui{p,1:7}] = deal(metset.load_dellow,'Edit',[1 1 1],[x(9)+bw*1/4 y(20) bw*3/4 eh],'qq_eval(''pn0'')',2,'metset.load_dellow');

% Search Buttons
p=p+1;[ui{p,1:7}] = deal('Show TIC','Pushbutton',colors.bc,[x(1) y(13) bw bh],'qq_load_cdf(4.1)',2,''); 
p=p+1;[ui{p,1:7}] = deal('Show Ions','Pushbutton',colors.bc,[x(1) y(14) bw bh],'qq_load_cdf(4.2)',2,''); 

updfcn='';
h0=qq_drawwin(winpar,ui,updfcn);

drawnow















   



        


















   






   






