function qq_plot_results(an)
%function qq_plot_results(an)
% an=0 draw chromatogram; update selected peak; update labels
% an=1 draw chromatogram
% an=2 update selected peak
% an=3 update labels

global XBL fileset metset CL RE fonts

% SJEKK FOR PLOTSCALE
h=findobj(gcf,'Tag','Plotcontrol'); if isempty(h) qq_plot_scale; end

if an==0
    plotresults; selupd; peaklabels
end
if an==1
    plotresults; 
end
if an==2
    selupd;
end
if an==3
    peaklabels;
end

% ----------------- PLOTRESULTS -----------------
function plotresults

global RE XBL fileset metset positions

n_peaks=fileset.results_npeaks;
selected=fileset.results_selpeak;

%switch metset.global_plotscale
%    case 1; fileset.chrom_plotlab=fileset.rt;
%    case 2; fileset.chrom_plotlab=fileset.results_rivect;
%    case 3; fileset.chrom_plotlab=1:length(fileset.rt)
%end

peakpl=subplot('position',positions.largeplot); 
plot(0); 
hold on

% Rectangles
le=length(fileset.chrom_plotlab);
for p=1:fileset.results_npeaks
    [maxv,maxp]=max(RE(p).C); [minv,minp]=min(RE(p).C);
               
    presentv=find(RE(p).C>maxv*0.05); 
    stp=RE(p).startp+min(presentv)-1; if stp<1 stp=1; end
    enp=RE(p).startp+max(presentv); if enp>length(fileset.chrom_plotlab) enp=length(fileset.chrom_plotlab); end
   
    rectvect=[fileset.chrom_plotlab(stp) minv fileset.chrom_plotlab(enp)-fileset.chrom_plotlab(stp) maxv*1.05];
      
    h=rectangle('position',rectvect,...
       'LineWidth',2,...
       'EdgeColor',[1 0.5 0],...
       'FaceColor',[0.9 0.9 0.9],...
       'Visible','off',...
       'Tag','Peakmarker',...
       'UserData',p);
end

fileset.chrom_tic3=zeros(1,length(fileset.rt)); % TIC of peaks

show_ri=0; h=findobj('Tag','CalibrCheck'); show_ri=get(h,'Value');
for p=1:fileset.results_npeaks
    [n_scans,d]=size(RE(p).C);
    xvect=(1:n_scans); 
    xvect=xvect+RE(p).startp-1;
    
    fileset.chrom_tic3(RE(p).startp:RE(p).endp)=fileset.chrom_tic3(RE(p).startp:RE(p).endp)+RE(p).C';
    h(p)=plot(fileset.chrom_plotlab(xvect),RE(p).C,'UserData',p,'ButtonDownFcn','qq_resultscall(16.3)'); 
    
    if RE(p).identified==1 set(h(p),'color',[0 0.6 0]); end % Identifisert topp er grnn
    if isnan(RE(p).ri_def)==0 & show_ri==1 set(h(p),'color',[1 0 0],'linewidth',1); end % Kalibreringstopper er rde
    if RE(p).clu_nr==0 set(h(p),'color',[1 0 0],'linewidth',2); end % Topper fra slettede clustre
end

%Residual TIC, Baseline subtracted tic (tic2) - tic of peaks( tic3)
fileset.chrom_tic4=fileset.chrom_tic2-fileset.chrom_tic3;
h2=plot(fileset.chrom_plotlab,fileset.chrom_tic4); set(h2,'color',[0.5 0.5 0.5]); 
hold off

% Heading p toppene
for p=1:fileset.results_npeaks
    text(fileset.chrom_plotlab(RE(p).maxp),max(RE(p).C),'*',...
        'Fontsize',8,'HorizontalAlignment','Center','Interpreter','none',...
        'VerticalAlignment','Bottom','Tag','Peaklabels','UserData',p);
end
    
% Peakinfotxt
str=[' '];text(0.03,0.95,str,'units','normalized','fontweight','bold','verticalalignment','top','Tag','Peakinfotxt','Interpreter','none'); 
set(gca,'Tag','Mainchrom','UserData','Results Chromatogram')
qq_copyfigure
%scale; yscale;
% Set slider
sp=fileset.chrom_cent/length(fileset.rt); sh=findobj(gcf,'Tag','Plotslider'); set(sh,'Value',sp*100);


% ----------- SELUPD -------------
function selupd

global fileset metset RE

% Peakinfo
str='';
if fileset.results_selpeak>0
    % Compatibility with old data (pre 11-04)
    if isfield(RE(fileset.results_selpeak),'compound_name') RE(fileset.results_selpeak).cmp_name = RE(fileset.results_selpeak).compound_name; end
    if isfield(RE(fileset.results_selpeak),'compound_code') RE(fileset.results_selpeak).cmp_shortname = RE(fileset.results_selpeak).compound_code; end
    if isfield(RE(fileset.results_selpeak),'cas_nr') RE(fileset.results_selpeak).cmp_code = RE(fileset.results_selpeak).cas_nr; end
    % optput
    str{1}=['Peak: ',num2str(fileset.results_selpeak) ' (cluster: ' num2str(RE(fileset.results_selpeak).clu_nr) ')'];
    str{2}=['Area: ',num2str(RE(fileset.results_selpeak).area)];
    str{3}=['Area pct: ',num2str(RE(fileset.results_selpeak).areapct)];
    str{4}=['Rt: ',num2str(RE(fileset.results_selpeak).rt)];
    str{5}=['RI: ',num2str(RE(fileset.results_selpeak).ri_calc)];     
    if RE(fileset.results_selpeak).identified==1
         str{6}=['Name: ',RE(fileset.results_selpeak).cmp_name];
         str{7}=['Short name: ',RE(fileset.results_selpeak).cmp_shortname];
         str{8}=['Reg Code: ',RE(fileset.results_selpeak).cmp_code];
         if isnan(RE(fileset.results_selpeak).ri_def)==0
             str{9}=['RI (def): ',num2str(RE(fileset.results_selpeak).ri_def)];
         end             
    end
    
end
h=findobj('Tag','Peakinfotxt'); set(h,'String',str);

% Rectangles
h=findobj('Tag','Peakmarker'); 
set(h,'Visible','off');
h=findobj(h,'UserData',fileset.results_selpeak);
set(h,'Visible','on')

h=findobj('Tag','PnTxt'); set(h,'String',num2str(fileset.results_selpeak));

%--------- PEAKLABELS ----------
% Oppdaterer peaklabels 
function peaklabels

global fileset metset RE

h=findobj('Tag','Peaklabels');
nr=get(h,'UserData'); if iscell(nr)==0 nr=num2cell(nr); end

for x=1:fileset.results_npeaks
    if metset.results_peaklabel<2  txt=num2str(RE(nr{x}).rt,4); end
    if metset.results_peaklabel==4 txt=num2str(RE(nr{x}).areapct,4); end
    if metset.results_peaklabel==3 txt=num2str(RE(nr{x}).area,4); end
    if metset.results_peaklabel==2 txt=num2str(RE(nr{x}).ri_calc,5); end
    set(h(x),'String',txt)
end
