function qq_plot_scale(an)

% function qq_plot_scale(an)
% 
% Function for manipulating chromatographic scale
% an==0: Draws zoomline
% 1>an>2: zoom and scaling
% an==100; updates zoomline and plot according to fileset
% an==101; updates plot and fileset according to zoomlline
% an==102; updates zoomline and fileset according to plot 
% an==103; redraw (change retention scale)

global fileset metset

if nargin==0; an=0; end

% -------------- DRAW LINE  ----------------
if an==0 zoomline; end

% Sets chromatogram as current axes
mainchrom=findobj(gcf,'Tag','Mainchrom');
%if isempty(mainchrom) disp('---!! axes not found ---'); an=-1; end
set(gcf,'CurrentAxes',mainchrom)

%ActiveChrom=get(gca,'UserData') % TEMP

% -------------- INITIALIZE  ----------------
if isfield(fileset,'chrom_li')==0; fileset.chrom_li=1; end
if isfield(fileset,'chrom_ri')==0; fileset.chrom_ri=length(fileset.rt); end
if isfield(fileset,'chrom_plotlab')==0; fileset.chrom_plotlab=fileset.rt; end
if isfield(fileset,'chrom_cent')==0; fileset.chrom_cent=round((fileset.chrom_li+fileset.chrom_ri)/2); end


% ----------------- ZOOM -----------------
% Initialize
if an>1 & an<2
    n_scans=length(fileset.rt);
    li=fileset.chrom_li; ri=fileset.chrom_ri;
    cp=(li+ri)/2; 
    zf=(ri-li+1)/n_scans;
end

% Zoom in
if an==1.1 
    zf=zf*0.5; range=round(n_scans*zf);
    li=cp-(range/2); li=ceil(li); ri=cp+(range/2); ri=floor(ri);
end
% Zoom out
if an==1.2 
    zf=zf*2; range=round(n_scans*zf);
    li=cp-(range/2); li=ceil(li); ri=cp+(range/2); ri=floor(ri);  
end  
% Slider
if an==1.3 
    slide=gcbo; value=get(slide,'Value');
    range=1+ri-li;
    cp=round(value);
    li=round(cp-(range/2))+1; 
    ri=round(cp+(range/2));
end

% Adjust by arrows
if an==1.35 
    kp=double(get(gcf,'CurrentCharacter'));
    if kp==28 
        %disp('-- left arrow --'); 
        step=round((ri-li)/5); cp=cp-step; li=li-step; ri=ri-step;
    end
    if kp==29 
        %disp('-- right arrow --'); 
        step=round((ri-li)/5); cp=cp+step; li=li+step; ri=ri+step;        
    end  
end

if an==1.5 % Set X manually
    h=findobj('Tag','Xscale_low_edit'); str=get(h,'String'); 
    xlow=str2double(str); xlow=xlow(1);
    ind=find(fileset.chrom_plotlab<=xlow); li=max(ind); 
    if isempty(li) li=1; end
    h=findobj('Tag','Xscale_high_edit'); str=get(h,'String'); 
    xhigh=str2double(str); xhigh=xhigh(1);
    ind=find(fileset.chrom_plotlab>=xhigh) ; ri=min(ind); 
    if isempty(ri) ri=length(fileset.chrom_plotlab); end
    if ri<li l=li; r=ri; li=r; ri=l; end
    if li==ri li=1; ri=length(fileset.chrom_plotlab); end
end

if an>1 & an<2
    if li<1 
        range=ri-li; if range>n_scans range=n_scans; end
        li=1; ri=1+range; 
    end; 
    if ri>n_scans; 
        range=ri-li; if range>n_scans range=n_scans; end
        ri=n_scans; li=ri-range; if li<1 li=1; end
    end
    if ri>li
        fileset.chrom_li=li; fileset.chrom_ri=ri; fileset.chrom_cent=round((li+ri)/2);
    end
    
    % UPDATE SLIDER 
    range=ri-li+1; winfrac=range/n_scans;
    maxv=ceil(n_scans-range/2); minv=floor(range/2); val=cp;
    if maxv<val maxv=val; end
    if minv>val minv=val; end
    if minv>=maxv minv=cp-1; maxv=cp+1; end
    if range==n_scans minv=cp-1; maxv=cp+1; val=cp; end
    sliderh=findobj('Tag','Plotslider');  
    set(sliderh,'Max',maxv,'Min',minv,'Value',val,'SliderStep',[winfrac/5 winfrac])    
    
    xscale; yscale     
end

% Y-SCALE
if an==3
    manualy
end
if an==3.1 % lock Y
    set(findobj('Tag','ASToggle'),'Value',0)
    set(findobj('Tag','YLockToggle'),'Value',1)
end
if an==3.2 % Autoscale Y
    set(findobj('Tag','YLockToggle'),'Value',0)
    set(findobj('Tag','ASToggle'),'Value',1)
    yscale
end

% Change scale (retention time/ index / scan no.)
if an==4
    h=findobj('Tag','ScaleRadio'); set(h,'Value',0); str=get(gcbo,'String'); 
    if strcmp(str,'Time') metset.global_plotscale=1; fileset.chrom_plotlab=fileset.rt; set(gcbo,'Value',1); end
    if strcmp(str,'Index') 
        if isfield(fileset,'results_rivect')==1
            metset.global_plotscale=2; fileset.chrom_plotlab=fileset.results_rivect; set(gcbo,'Value',1);
        else
            qq_message(1,'|r|Restention indices not calculated')
            if metset.global_plotscale==1; h=findobj('Tag','ScaleRadio','String','Time'); set(h,'Value',1); end
            if metset.global_plotscale==3; h=findobj('Tag','ScaleRadio','String','Scan No.'); set(h,'Value',1); end
        end
    end
    if strcmp(str,'Scan No.') metset.global_plotscale=3; fileset.chrom_plotlab=1:length(fileset.rt); set(gcbo,'Value',1); end
    redraw
end

% Change retention time
if an>5 & an<6
    h=findobj('Tag','MultiplierEdit'); str=get(h,'String'); val=str2double(str);
    if isnan(val) val=1; set(h,'String','1'); end
    if an==5.1 fileset.rt=fileset.rt*val; end
    if an==5.2 fileset.rt=fileset.rt/val; end
    fileset.chrom_plotlab=fileset.rt;
    h=findobj('Tag','ScaleRadio'); set(h,'Value',0);
    h=findobj('Tag','ScaleRadio','String','Time'); set(h,'Value',1);
    redraw
end

% ---- UPDATES----
if an==100
    readfileset; yscale;
end

if an==103
    redraw; 
end



% ====================================================================================
%                                   SUBFUNCTIONS
% ====================================================================================

% ---------------- MANUALY ------------------
function manualy

Yscale_low_edit=findobj('Tag','Yscale_low_edit');
Yscale_high_edit=findobj('Tag','Yscale_high_edit');
Yscale_exp_edit=findobj('Tag','Yscale_exp_edit');
YLockToggle=findobj(gcf,'Tag','YLockToggle'); set(YLockToggle,'Value',1);
ASToggle=findobj(gcf,'Tag','ASToggle'); set(ASToggle,'Value',0);
minymaxy=get(gca,'Ylim'); ymin_old=minymaxy(1); ymax_old=minymaxy(2);

ymi=str2double(get(Yscale_low_edit,'String')); ymi=ymi(1);
yma=str2double(get(Yscale_high_edit,'String')); yma=yma(1);
exp=str2double(get(Yscale_exp_edit,'String')); exp=exp(1);
ymin=ymi*10^exp; 
ymax=yma*10^exp;
if sum(isnan([ymin ymax]))==0
    ylim([ymin ymax]);
else
    ba=10^floor(log10(ymax_old)); yma=round(100*ymax_old/ba)/100; ymi=round(100*ymin_old/ba)/100;
    set(Yscale_low_edit,'String',num2str(ymi))
    set(Yscale_high_edit,'String',num2str(yma))
    set(Yscale_exp_edit,'String',num2str(log10(ba)))  
end

% ---------------- REDRAW ------------------
function redraw


wintag=get(gcf,'Tag'); plottag=get(gca,'Tag');

if strmatch(plottag,'Mainchrom') & strmatch(wintag,'MainWin')
    qq_plot_mainchrom(0); 
end
if strmatch(plottag,'Mainchrom') & strmatch(wintag,'BlWin')
    qq_plot_blchrom(0); 
end
if strmatch(plottag,'Mainchrom') & strmatch(wintag,'PeakWin')
    qq_plot_peakchrom(0); 
end
if strmatch(plottag,'Mainchrom') & strmatch(wintag,'ResultsWin')
    qq_plot_results(0); 
end

xscale; yscale;

% ---------------- READFILESET ------------------
function readfileset
% update according to fileset
global fileset

if isfield(fileset,'chrom_li')==0 fileset.chrom_li=1; end 
if isfield(fileset,'chrom_ri')==0 fileset.chrom_ri=length(fileset.rt); end
if isfield(fileset,'chrom_cent')==0 fileset.chrom_cent=round(mean(fileset.chrom_li:fileset.chrom_ri)); end

chromh=findobj(gcf,'Tag','Mainchrom');

set(chromh,'Xlim',[fileset.chrom_plotlab(fileset.chrom_li) fileset.chrom_plotlab(fileset.chrom_ri)]) 

% Tresholdslider ny kode
range=1+fileset.chrom_ri-fileset.chrom_li; % Range of window shown
n_scans=length(fileset.rt); winfrac=range/n_scans; % Fraction of scale covered by current view
cp=(fileset.chrom_ri+fileset.chrom_li)/2;
maxv=ceil(n_scans-range/2); minv=floor(range/2); val=cp;
if maxv<val maxv=val; end
if minv>val minv=val; end
if minv>=maxv minv=cp-1; maxv=cp+1; end
if range==n_scans minv=cp-1; maxv=cp+1; val=cp; end
sliderh=findobj('Tag','Plotslider');  
set(sliderh,'Max',maxv,'Min',minv,'Value',val,'SliderStep',[winfrac/5 winfrac])


% Edits (X)
h=findobj('Tag','Xscale_low_edit'); set(h,'string',num2str(fileset.chrom_plotlab(fileset.chrom_li),'%11.4g'))
h=findobj('Tag','Xscale_high_edit'); set(h,'string',num2str(fileset.chrom_plotlab(fileset.chrom_ri),'%11.4g'))

% ---------------- YSCALE ------------------
function yscale
global fileset RE XBL

wintag=get(gcf,'Tag');
plottag=get(gca,'Tag');
Yscale_low_edit=findobj('Tag','Yscale_low_edit');
Yscale_high_edit=findobj('Tag','Yscale_high_edit');
Yscale_exp_edit=findobj('Tag','Yscale_exp_edit');
YLockToggle=findobj(gcf,'Tag','YLockToggle');
ASToggle=findobj(gcf,'Tag','ASToggle');

identified=0;
% Select tic to calculate min/max from
if strmatch(plottag,'Mainchrom') & strmatch(wintag,'MainWin')
    hightrace=fileset.chrom_tic1;
    lowtrace=fileset.chrom_tic2;
    identified=1;
end
if strmatch(plottag,'Mainchrom') & strmatch(wintag,'BlWin')
    hightrace=fileset.chrom_tic3;
    lowtrace=fileset.chrom_tic3;
    identified=1;
end
if strmatch(plottag,'Mainchrom') & strmatch(wintag,'PeakWin')
    hightrace=fileset.chrom_tic2;
    lowtrace=fileset.chrom_tic2;
    identified=1;
end
if strmatch(plottag,'Mainchrom') & strmatch(wintag,'ResultsWin')
    hightrace=fileset.chrom_tic2;
    lowtrace=fileset.chrom_tic4;
    identified=1;
end
if strmatch(plottag,'Mainchrom') & strmatch(wintag,'UCMWin')
    hightrace=fileset.chrom_tic5;
    lowtrace=fileset.chrom_tic6;
    identified=1;
end

if identified
    
    % IF AUTOSCALE
    val=get(YLockToggle,'Value');
    if val==0
        set(ASToggle,'Value',1);
        ymax=max(hightrace(fileset.chrom_li:fileset.chrom_ri));
        ymin=min(lowtrace(fileset.chrom_li:fileset.chrom_ri));
        if ymin>ymax; [ymin,ymax]=deal(ymax,ymin); end
        o=(ymax-ymin)*0.02; ymin=ymin-o; ymax=ymax+3*o;
        if ymin==ymax ymin=ymin-1; ymax=ymax+1; end
        set(gca,'Ylim',[ymin ymax]);
    
        % Sets values in edit
        ba=10^floor(log10(ymax)); yma=round(100*ymax/ba)/100; ymi=round(100*ymin/ba)/100;
        set(Yscale_low_edit,'String',num2str(ymi))
        set(Yscale_high_edit,'String',num2str(yma))
        set(Yscale_exp_edit,'String',num2str(log10(ba)))    
    else
        ymi=str2double(get(Yscale_low_edit,'String')); ymi=ymi(1);
        yma=str2double(get(Yscale_high_edit,'String')); yma=yma(1);
        b=str2double(get(Yscale_exp_edit,'String')); ba=10^b(1);
        ymin=ymi*ba; ymax=yma*ba; o=(ymax-ymin)*0.02;
        set(gca,'Ylim',[ymin ymax]);
    end

    % Peaknumbers/Rank in peak chrom
    if strmatch(get(gcf,'Tag'),'PeakWin')     
    % Peak numbers
        h=findobj('Tag','PnrTxt'); n=length(h); vp=ymax-1*o;
        for x=1:n pos=get(h(x),'Position'); set(h(x),'Position',[pos(1) vp pos(3)]); end
        % Rank
        h=findobj('Tag','RankMark'); n=length(h); vp=ymax-3*o;
        for x=1:n pos=get(h(x),'Position'); set(h(x),'Position',[pos(1) vp pos(3)]); end    
    end 

end


% ---------------- XSCALE ------------------
% Setter x-skala
function xscale 
global fileset

%disp('---- qq_plot_scale (xscale) ----')

%set(gca,'Tag','Chromatogram');
xlim([fileset.chrom_plotlab(fileset.chrom_li) fileset.chrom_plotlab(fileset.chrom_ri)]);

h=findobj('Tag','Xscale_low_edit'); set(h,'string',num2str(fileset.chrom_plotlab(fileset.chrom_li),'%11.4g'))
h=findobj('Tag','Xscale_high_edit'); set(h,'string',num2str(fileset.chrom_plotlab(fileset.chrom_ri),'%11.4g'))



% -------------- SUBFUNCTION ZOOMLINE -------------------
function zoomline

global fonts positions colors metset

if isfield(metset,'global_plotscale')==0 metset.global_plotscale=1; end

bw=positions.bw;
bh=positions.bh;
eh=positions.eh;
x=positions.gl:positions.gw:1; % horisontalt grid
y=positions.gt+0.5:-positions.gh:0; % vertikalt grid
o=bh-eh;
h0=gcf;

% ZOOMLINE
h=findobj(gcf,'Tag','Plotcontrol'); 
if isempty(h)
    uicontrol('Parent',h0,'Callback','qq_plot_scale(1.1)','Position',[x(1) y(12)+o bw/4 eh],'String','+','Tag','Plotcontrol','FontSize',fonts(5));
    uicontrol('Parent',h0,'Callback','qq_plot_scale(1.2)','Position',[x(1)+bw/4 y(12)+o bw/4 eh],'String','-','Tag','Plotcontrol','FontSize',fonts(5));
    uicontrol('Parent',h0,'Callback','qq_plot_scale(1.3)','Max',100,'SliderStep',[0.02 0.20],'Position',[x(1)+bw/2 y(12)+o bw*1.5 eh], ...
    	'Style','slider','Tag','Plotslider','Value',50.0);
    % Xscale
    uicontrol('Parent',h0,'Callback','','Position',[x(3) y(12)+o bw/5 eh], ...
    	'Style','Text','Tag','','Value',0,'String','X','HorizontalAlignment','center');
    uicontrol('Parent',h0,'Callback','qq_plot_scale(1.5)','Position',[x(3)+bw*1/5 y(12)+o bw*2/5 eh], ...
    	'Style','Edit','Tag','Xscale_low_edit','Value',0,'String','--','FontSize',10,'FontName','FixedWidth','FontWeight','Normal');
    uicontrol('Parent',h0,'Callback','qq_plot_scale(1.5)','Position',[x(3)+bw*3/5 y(12)+o bw*2/5 eh], ...
    	'Style','Edit','Tag','Xscale_high_edit','Value',0,'String','--','FontSize',10,'FontName','FixedWidth','FontWeight','Normal');
    % Yscale
    uicontrol('Parent',h0,'Callback','','Position',[x(4) y(12)+o bw/5 eh], ...
    	'Style','Text','Tag','','String','Y','HorizontalAlignment','center');    
    uicontrol('Parent',h0,'Callback','qq_plot_scale(3)','Position',[x(4)+bw*1/5 y(12)+o bw*2/5 eh], ...
    	'Style','Edit','Tag','Yscale_low_edit','String','--','FontSize',10,'FontName','FixedWidth','FontWeight','Normal');
    uicontrol('Parent',h0,'Callback','qq_plot_scale(3)','Position',[x(4)+bw*3/5 y(12)+o bw*2/5 eh], ...
    	'Style','Edit','Tag','Yscale_high_edit','String','--','FontSize',10,'FontName','FixedWidth','FontWeight','Normal');
    uicontrol('Parent',h0,'Callback','','Position',[x(4)+3*bw/3 y(12)+o bw/6 eh], ...
    	'Style','Text','Tag','','String','e');
    uicontrol('Parent',h0,'Callback','qq_plot_scale(3)','Position',[x(4)+3*bw/3+bw/6 y(12)+o bw*0.25 eh], ...
    	'Style','Edit','Tag','Yscale_exp_edit','String','-','FontSize',10,'FontName','FixedWidth','FontWeight','Normal');
   % Auto/Manual Y toggle buttons 
   uicontrol('Parent',h0,'Callback','qq_plot_scale(3.1)','Position',[x(5)+bw*0.5 y(12)+o bw/3 eh], ...
    	'Style','togglebutton','Tag','YLockToggle','Value',0,'String','Lock');
   uicontrol('Parent',h0,'Callback','qq_plot_scale(3.2)','Position',[x(5)+bw*0.5+bw/3 y(12)+o bw/3 eh], ...
    	'Style','togglebutton','Tag','ASToggle','Value',1,'String','Auto');   
   % Select scale:
    uicontrol('Parent',h0,'Callback','','Position',[x(6)+bw/3 y(12)+o bw eh], ...
    	'Style','Text','Tag','','Value',1,'String','Retention scale:');
    uicontrol('Parent',h0,'Callback','qq_plot_scale(4)','Position',[x(7)+bw/3 y(12)+o bw/2 eh], ...
    	'Style','RadioButton','Tag','ScaleRadio','Value',1,'String','Time'); 
    uicontrol('Parent',h0,'Callback','qq_plot_scale(4)','Position',[x(7)+bw/3+bw/2 y(12)+o bw/2 eh], ...
    	'Style','RadioButton','Tag','ScaleRadio','Value',1,'String','Index'); 
    uicontrol('Parent',h0,'Callback','qq_plot_scale(4)','Position',[x(7)+bw/3+bw y(12)+o bw eh], ...
    	'Style','RadioButton','Tag','ScaleRadio','Value',1,'String','Scan No.'); 
    
    % Retention time multiplier
    %uicontrol('Parent',h0,'Callback','qq_plot_scale(5.1)','Position',[x(9) y(12)+o bw/2 eh], ...
    %	'Style','Pushbutton','Tag','','Value',1,'String','Multipl Rt');  
    %uicontrol('Parent',h0,'Callback','','Position',[x(9)+bw/2 y(12)+o bw/4 eh], ...
    % 	'Style','Edit','Tag','MultiplierEdit','String','60','HorizontalAlignment','center','FontSize',10,'FontName','FixedWidth','FontWeight','Normal');  
    % uicontrol('Parent',h0,'Callback','qq_plot_scale(5.2)','Position',[x(9)+bw*3/4 y(12)+o bw/2 eh], ...
    % 	'Style','Pushbutton','Tag','','Value',1,'String','Div Rt');  
    
   
    h=findobj('Tag','ScaleRadio'); set(h,'Value',0)
    switch metset.global_plotscale
        case 1; h=findobj(h,'String','Time'); set(h,'Value',1); 
        case 2; h=findobj(h,'String','Index'); set(h,'Value',1); 
        case 3; h=findobj(h,'String','Scan No.'); set(h,'Value',1);          
    end 
    
end
% TICLINE
%h=findobj(gcf,'Tag','TICcontrol'); xp=8;
%if isempty(h) & [strmatch(get(gcf,'Tag'),'PeakWin') strmatch(get(gcf,'Tag'),'BlWin')]; 
%    uicontrol('Parent',h0,'Callback','qq_plot_scale(2.1)','Position',[x(xp) y(12)+o bw/2 eh],'String','TIC','Tag','TICcontrol');
%    uicontrol('Parent',h0,'Callback','qq_plot_scale(2.2)','Position',[x(xp)+bw*0.5 y(12)+o bw/2 eh],'String','Log TIC','Tag','TICcontrol');
%    uicontrol('Parent',h0,'Callback','qq_plot_scale(2.3)','Position',[x(xp)+bw y(12)+o bw/2 eh],'String','Var.Vin','Tag','TICcontrol');
%    uicontrol('Parent',h0,'Callback','qq_plot_scale(2.4)','Position',[x(xp)+bw*1.5 y(12)+o bw/2 eh],'String','Smooth','Tag','TICcontrol');
%end
% RI-scale

%h=findobj(gcf,'Tag','RIScaleCheck'); xp=9;
%if isempty(h) & strmatch(get(gcf,'Tag'),{'MainWin','ResultsWin'})
%    uicontrol('Parent',h0,'Callback','qq_plot_scale(103)','Position',[x(xp)+bw/2 y(12)+o bw eh],'Style','Checkbox','String','Use RI-Scale','Tag','RIScaleCheck');
%end
