function [X,rt,ions,info]=qq_readms(fid)
% READS HP (AGILENT) MS-CHEMSTATION FILES INTO MATLAB
%
% X    = Raw data
% rt   = Retention time
% ions = mass numbers
% info = Structure 
% fid  = file identifier to DATA.MS (open with fopen, remember to close) 
%
% Svein Are Mjs jan. 2005: svein.mjos@fiskeriforskning.no

massoff=0.1; % Mass offset

%fseek(fid, 5, 'bof'); A = fread(fid, 19, 'schar'); char(A') % Header
fseek(fid, 25, 'bof'); A = fread(fid, 61, 'schar')'; mina=min(find(A~=32)); info.Sample_Name=deblank(char(A(mina:length(A)))); % Data Name
fseek(fid, 149, 'bof'); A = fread(fid, 29, 'schar'); info.Operator=deblank(char(A')); % Operator
fseek(fid, 179, 'bof'); A = fread(fid, 29, 'schar'); info.Time=deblank(char(A')); % Date/Time
fseek(fid, 209, 'bof'); A = fread(fid, 9, 'schar'); info.Instrument=deblank(char(A')); % Instr
fseek(fid, 229, 'bof'); A = fread(fid, 19, 'schar'); info.Method=deblank(char(A')); % Instr
fseek(fid, 87, 'bof'); A = fread(fid, 61, 'schar')'; mina=min(find(A~=32)); info.Note=deblank(char(A(mina:length(A)))); 
info.Bottle_No=readshort(fid,254);
info.N_scans=readlong(fid,278);
info.First_rt=readlong(fid,282)/60000; % konverterer fra ms til min.
info.Last_rt=readlong(fid,286)/60000; % konverterer fra ms til min.

%data_offset=(readlong(fid,264)*2)-2 % Data offset (words * 2 bytes)
%dir_offset=(readlong(fid,260)*2)-2 % Data offset (words * 2 bytes)

ind=getindices(fid); % Returns adresses of each spectral record
rt=getrt(fid); 
% tic
n_scans=length(ind); X=[];
for x=n_scans:-1:1 % Reverse to avoid gradually expansion of X
    [mz,abund]=getspec(fid,ind(x)); % Returns masses and abundance for spectr. no. 'ind(x)'
    mz=mz-massoff; mz=round(mz); % Converts masses to integers
    if length(mz)>length(unique(mz)); [mz,abund]=sumequal(mz,abund); end % If several masses are rounded to the same integer
    X(x,mz)=abund; % Builds X-matrix
end
%disp(['----- ELAPSED: ' num2str(toc) ' -----'])
[m,n]=size(X);
ions=1:n;

% Trunkerer
sumspec=(sum(X)); delions=find(sumspec==0); ions(delions)=[]; X(:,delions)=[];


% -------------------------------------------------------------
% ------------------------- SUBFUNCTIONS ----------------------
% -------------------------------------------------------------


% ------------------------- GETSPEC ----------------------
function [mz,abund]=getspec(fid,ind)
% Index er start p spectral record
% Returnerer vektorer med masser og tilsvarende abundance

mz=[];abund=[]; % TEMP
n_peaks=readshort(fid,ind+12); p=0;
for x=1:n_peaks
    mz(x)=readshort(fid,ind+18+p)/20; % Masse
    % ----------------------
    fseek(fid, ind+20+p, 'bof'); lo=fread(fid,2);
    scalebytes=find([bitget(lo(1),7:8)]); % Scale bytes 15-16
    scale=sum(bitset(0,scalebytes));
    mantbytes=find([bitget(lo(2),1:8) bitget(lo(1),1:6)]); % Mantissa er byte 1-14
    mant=sum(bitset(0,mantbytes));
    abund(x)=mant*(8^scale); % Konvertering til abundance, se Chemstation manual s.400
    % ----------------------
    p=p+4;
end
mz=fliplr(mz); abund=fliplr(abund);


% ------------------------- GETINDICES ----------------------
function ind=getindices(fid)
% Returnerer adressene til hver spectral record
dir_offset=(readlong(fid,260)*2)-2; % Data offset (words * 2 bytes)
n_scans=readlong(fid,278); % Antall scan
p=dir_offset;
for x=1:n_scans ind(x)=(readlong(fid,p)*2)-2; p=p+12; end



% ------------------------- GETRT ----------------------
function rt=getrt(fid)
% Returnerer retensjonstid (min) til hver spectral record
dir_offset=(readlong(fid,260)*2)-2; % Data offset (words * 2 bytes)
n_scans=readlong(fid,278); % Antall scan
p=dir_offset;
for x=1:n_scans  rt(x)=(readlong(fid,p+4))/60000;  p=p+12; end



% ------------------------- GETTIC ----------------------
function tic=gettic(fid)
% Returnerer retensjonstid (min) til hver spectral record
dir_offset=(readlong(fid,260)*2)-2; % Data offset (words * 2 bytes)
n_scans=readlong(fid,278); % Antall scan
p=dir_offset;
for x=1:n_scans tic(x)=readlong(fid,p+8); p=p+12; end



% ------------------------- READLONG ----------------------
% read long integer from position=pos
function val=readlong(fid,pos)
%disp('---- READLONG ----')
fseek(fid, pos, 'bof'); lo=fread(fid,4);
bytes=find([bitget(lo(4),1:8) bitget(lo(3),1:8) bitget(lo(2),1:8) bitget(lo(1),1:8)]);
val=sum(bitset(0,bytes));


% ------------------------- READSHORT ----------------------
% read short integer from position=pos
function val=readshort(fid,pos)
%disp('---- READLONG ----')
fseek(fid, pos, 'bof'); lo=fread(fid,2);
bytes=find([bitget(lo(2),1:8) bitget(lo(1),1:8)]);
val=sum(bitset(0,bytes));

% ------------------------- SUMEQUAL ----------------------
% Sums abundance of equal masses in mz, returns unique mz
function [mz,abund]=sumequal(mz,abund)
b=unique(mz);
for y=1:length(b)
    o=find(mz==b(y));
    tabund(y)=sum(abund(o));  
end
mz=b; abund=tabund; 
