function [X]=qq_transform(X,t)

% function [X]=transform(X,t)
% Transforms raw data X
% t=Type
%    Type 1: Transformation A 
%    Type 2: Transformation B
%    Type 3: Transformation C
%    Type 4: Transformation D
%    Type 5: Transformation E (Karjalainen)
%    Type 6: Normalization
%    Type 8: Transformation F (Keller)
%    Type 9: Transformation G (RAFA) 

dim=size(X);

% SETTINGS FOR TRANSFORM E
dwt=100; % Dwelltime i ms for SIM-analyser (millisekunder)
itl=14.1; % Inter-ion time lag, 14.1 ms for HP-5972 i SIM-mode (lave masser)
sst=45.4; % Scan setup-time, 45.4 ms (59.5-14.1) for HP-5972 i SIM-mode (lave masser)
% NB! Kun ndvendig med EN verdi? ITL som prosent av SST ??
% DENNE VERDIEN KAN TUNES P RENE TOPPER !!!



% SETTINGS FOR NORMALISERING
norm=1; % 1: Samme sum, 2: Samme max


if t==1
   % Hvert ion divideres p ionet med lavere masse.
   % Ionet med lavest masse i hvert scan settes lik 1.
   XA=ones(dim);
   for i1 = 1 : dim(1)
      for i2 = 2 : dim(2)
         XA(i1,i2) = X(i1,i2)/X(i1,i2-1);     
      end
   end
   XA = origprof(XA,X);
   X=XA;
end % if

if t==2
   % Hvert ion divideres p ionet med hyere masse.
   % Ionet med hyest masse i hvert scan settes lik 1.
   XB=ones(dim);
   for i1 = 1 : dim(1)
      for i2 = 1 : (dim(2)-1)
         XB(i1,i2) = X(i1,i2)/X(i1,i2+1);     
      end
   end   
   XB = origprof(XB,X);
   X=XB;
end

if t==3
   % Hvert ion divideres p ionet med lavere masse (scannet etter).
   % Ionet med lavest masse i hvert scan divideres med ionet med hyest masse i scannet etter
   XC=ones(dim);
   for i1 = 1 : dim(1)-1
      for i2 = 2 : dim(2)
         XC(i1,i2) = X(i1,i2)/X(i1,i2-1);
      end
      XC(i1,1)  = X(i1,1)/X(i1+1,dim(2));
   end
   % XC(i1+1,:)=[]; % Stryker siste             
   XC(i1+1,:)=XC(i1,:); % Setter siste lik nest siste   
   XC = origprof(XC,X);               
   X=XC;
end

if t==4
   % Hvert ion divideres p ionet med hyere masse (scannet fr).
   % Ionet med hyest masse i hvert scan divideres med ionet med lavest masse i scannet fr
   XD=ones(dim);
   for i1 = 2 : dim(1)
      for i2 = 1 : dim(2)-1
         XD(i1,i2) = X(i1,i2)/X(i1,i2+1);
      end
      XD(i1,dim(2))= X(i1,dim(2))/X(i1-1,1);
   end
   % XD(1,:)=[];  % Stryker frste
   XD(1,:)=XD(2,:);  % Setter frste lik andre
   XD = origprof(XD,X);     
   X=XD;
end

if t==5
   % FIRST ORDER CORRECTION (KELLER/KARJALAINEN)
   it=dwt+itl;         % Tiden fra start av et ion til start av neste
   st=(it*dim(2))+sst; % Lengden til et scan: antall ioner * it + sst
   clear sig; clear XR

   for i=1:dim(1)-1
      time=[i;i+1];                            % Vektor med aktuelle tider
      for j=dim(2):-1:1                        % Ionet med hyest masse scannes frst
         sig(:,1)=X(time,j);                   % Vektor med signaler, tider time og ion j
         b=(polyfit(time,sig,1))';             % Beregner regr.-koeffisienter
         difft=((dim(2)-j)*it)/st;             % beregner diff i tid rel til frste ion i scannet. 
         diffs=((i+difft)*b(1)+b(2)) - (i*b(1)+b(2)); % signaldifferanse
         XE(i,j)=X(i,j)-diffs;                 % justerer ved  trekke fra signaldifferansen            
      end   
   end
   lastspec=X(i+1,:); 
   XE=[XE; lastspec];    % Legg til siste to ganger
   X=XE;
end

if t==8
   % SECOND ORDER CORRECTION (KELLER)
   it=dwt+itl;         % Tiden fra start av et ion til start av neste
   st=(it*dim(2))+sst; % Lengden til et scan: antall ioner * it + sst
   clear sig; clear XR

   for i=2:dim(1)-1
      time=[i-1;i;i+1];                            % Vektor med aktuelle tider
      for j=dim(2):-1:1                            % Ionet med hyest masse scannes frst
         sig(:,1)=X(time,j);                       % Vektor med signaler, tider time og ion j
         b=(polyfit(time,sig,2))';                 % Beregner regr.-koeffisienter
         difft=((dim(2)-j)*it)/st;                 % beregner diff i tid rel til frste ion i scannet. 
         diffs=(((i+difft)^2*b(1))+((i+difft)*b(2))+b(3)) - (i^2*b(1)+i*b(2)+b(3)); % signaldifferanse
         XF(i,j)=X(i,j)-diffs;                     % justerer ved  trekke fra signaldifferansen            
      end   
   end
   lastspec=X(i+1,:); 
   XF=[XF; lastspec];    % Legg til siste to ganger
   XF(1,:)=X(1,:);
   X=XF;
   %size(X)
   %size(XF)
end

% Normalisering
if t==6   
   XN=normalize(X,norm);
   XN = origprof(XN,X);
   X=XN;
end

%if t==8
%    X=(lsqstrip(X'))';
%end

if t==9
     X=(rafastrip(X,rank(X)));    
end


% ----------------------------- RAFASTRIP --------------------------------
function X_out=rafastrip(X_in,rnk)
% Fjerner scaneffekt ved hjelp av RAFA
% X_in: Inndata, C i rader, S i kolonner
% rnk=rank, settes hyere enn antatt rank

[n_scans,n_ions]=size(X_in);
c1=sum(X_in'); % Totalsignal TIC
c2=gradient(c1); % Deriverte til totalsignalet

s1=mean(X_in); % Gjennomsnittspekter
linspec=linspace(-1,1,n_ions); % linert spekter
s2=s1.*linspec; % Beregner spekteret til scan-effekten

%X_in=X_in'; 
c2=c2'; s2=s2'; % Transponerer for  passe med 'gammel' RAFA

% RAFA etter Lorber:
[U S V]=svds(X_in,rnk); % rnk: rank for decomposition (set higher than expected rank)
L=S^2; R=U*S; 
N=c2*s2';
a=[R*inv(L)]'*c2;
b=V'*s2;
t=1/(a'*b);

X_strip=t*c2*s2';
X_out=(X_in-X_strip);

% ----------------------------- LSQSTRIP --------------------------------
function X_out=lsqstrip(X_in)
% X_in: Inndata, C i rader, S i kolonner

c1=sum(X_in); % Totalsignal TIC
c2=gradient(c1); % Deriverte til totalsignalet
C=[c1;c2]; % C-matrise
S=X_in/C; % Estimerer spektra (minste kvadraters)
s2=S(:,2); % s til scaneffekten
X_strip=c2'*s2'; X_strip=X_strip'; % Beregner x til scan-effekt
X_out=(X_in-X_strip);


% ------------------------------------------------------------------------

function CX = origprof(IX,MX)
% Korreksjon til samme totalporofil (kromatografisk) som MX, 
% IX er profil som korrigeres
dim=size(IX);
for i1 = 1 : dim(1)
      sumMX     = sum(MX(i1,:)); % Beregner Totalprofil originale data
      sumIX    = sum(IX(i1,:));  % Beregner totalprofil transformerte data
      W        = sumMX/sumIX;    % Beregner vekter for korreksjon
      IX(i1,:) = IX(i1,:) .* W;  % Korrigerer
end
CX=IX;

% ------------------------------------------------------------------------

function NX = normalize(IX,type)   % SUBFUNCTION FOR VEKTING
% Vekting av variabler 
NX=IX;
dim=size(NX);
for i2 = 1 : dim(2)
    if type == 1 
        NX(:,i2)= NX(:,i2) * (100/sum(NX(:,i2)));
    end
    if type == 2
        NX(:,i2)= NX(:,i2) * (100/max(NX(:,i2)));
    end
end




